﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace TRsFuego
{
    class MyUtils
    {
        public static int clamp(int p, int l, int h)
        {
            p = (p < l) ? l : (p > h) ? h : p;

            return p;
        }
        public static int clampD(double p, int l, int h)
        {
            p = (p < l) ? l : (p > h) ? h : p;

            return (int)p;
        }
        public static Point clampP(Point p, Rectangle r)
        {
            p.X = (p.X < r.Left) ? r.Left : (p.X > r.Right) ? r.Right : p.X;
            p.Y = (p.Y < r.Top) ? r.Top : (p.Y > r.Bottom) ? r.Bottom : p.Y;
            return p;
        }
        public static Rectangle clampR(Rectangle p, Rectangle r)
        {
            int left = (p.X < r.Left) ? r.Left : (p.X > r.Right) ? r.Right : p.X;
            int top = (p.Y < r.Top) ? r.Top : (p.Y > r.Bottom) ? r.Bottom : p.Y;
            int right = (p.Right < r.Left) ? r.Left : (p.Right > r.Right) ? r.Right : p.Right;
            int bottom = (p.Bottom < r.Top) ? r.Top : (p.Bottom > r.Bottom) ? r.Bottom : p.Bottom;
            return new Rectangle(left, top, right - left, bottom - top);
        }
        public static double Pythag(double p1, double p2)
        {
            return Math.Sqrt(p1 * p1 + p2 * p2);
        }

        public static double PythagP(Point p1, Point p2)
        {
            return Math.Sqrt((p2.X - p1.X) * (p2.X - p1.X) + (p2.Y - p1.Y) * (p2.Y - p1.Y));
        }


        public static Point Centroid(Point[] source)
        {
            Point tmp = new Point();
            if (source.Length == 0) return tmp;
            foreach (Point i in source)
            {
                tmp.X += i.X;
                tmp.Y += i.Y;
            }
            tmp.X /= source.Length;
            tmp.Y /= source.Length;
            return tmp;
        }

        public static PointF CatmullRom(PointF p0, PointF p1, PointF p2, PointF p3, float t)
        {
            PointF ret = new PointF();

            float t2 = t * t;
            float t3 = t2 * t;

            ret.X = 0.5f * ((2.0f * p1.X) + (-p0.X + p2.X) * t +
                (2.0f * p0.X - 5.0f * p1.X + 4 * p2.X - p3.X) * t2 +
                (-p0.X + 3.0f * p1.X - 3.0f * p2.X + p3.X) * t3);

            ret.Y = 0.5f * ((2.0f * p1.Y) + (-p0.Y + p2.Y) * t +
                (2.0f * p0.Y - 5.0f * p1.Y + 4 * p2.Y - p3.Y) * t2 +
                (-p0.Y + 3.0f * p1.Y - 3.0f * p2.Y + p3.Y) * t3);

            return ret;
        }

        public static PointF Bezier(PointF p0, PointF p1, PointF p2, PointF p3, float t)
        {

            PointF p4 =new PointF(p1.X * t + p0.X - p0.X * t, p1.Y * t + p0.Y - p0.Y * t);
            PointF p5 =new PointF(p2.X * t + p1.X - p1.X * t, p2.Y * t + p1.Y - p1.Y * t);
            PointF p6 =new PointF(p3.X * t + p2.X - p2.X * t, p3.Y * t + p2.Y - p2.Y * t);

            PointF p7 =new PointF(p5.X * t + p4.X - p4.X * t, p5.Y * t + p4.Y - p4.Y * t);
            PointF p8 =new PointF(p6.X * t + p5.X - p5.X * t, p6.Y * t + p5.Y - p5.Y * t);

            return new PointF(p8.X * t + p7.X - p7.X * t, p8.Y * t + p7.Y - p7.Y * t); ;
        }
    }

 }
